#include <stdlib.h>

#include "gtcblist.h"

/************************************************************************\
*
* GtCBList class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Type of list elements */
typedef struct GtCBL
{
	GtSLink lnk;
	GtCallB cb;
	GtUDPtr ud;
} GtCBL;

/************************************************************************\
*
* Setup cblist.
*
\************************************************************************/
GtCBList::GtCBList(void)
{
	GtSInit(&hd);
}

/************************************************************************\
*
* Destroy cblist.
*
\************************************************************************/
GtCBList::~GtCBList(void)
{
	GtCBL *cbl;
	
	while((cbl=(GtCBL *)GtSRemHead(&hd)))
		free(cbl);
}

/************************************************************************\
*
* Add callback to list and return identifier.
*
\************************************************************************/
int GtCBList::AddCB(GtCallB cb,GtUDPtr ud)
{
	GtCBL *cbl;
	
	if((cbl=(GtCBL *)malloc(sizeof(GtCBL))))
	{
		cbl->cb=cb;
		cbl->ud=ud;
		GtSAddTail(&hd,&cbl->lnk);
		return((int)cbl);
	}
	return(0);
}

/************************************************************************\
*
* Remove callback from list.
*
\************************************************************************/
void GtCBList::RemCB(GtCallB cb)
{
	GtCBL *cbl,*tmp;
	
	for(cbl=(GtCBL *)GtSFirst(&hd); cbl; cbl=tmp)
	{
		tmp=(GtCBL *)GtSNext(&cbl->lnk);
		if(cbl->cb==cb)
		{
			GtSRemove(&hd,&cbl->lnk);
			free(cbl);
			return;
		}
	}
}

/************************************************************************\
*
* Remove callback from list according to identifier. 
*
\************************************************************************/
void GtCBList::RemCB(int id)
{
	GtCBL *cbl,*tmp;
	
	for(cbl=(GtCBL *)GtSFirst(&hd); cbl; cbl=tmp)
	{
		tmp=(GtCBL *)GtSNext(&cbl->lnk);
		if((int)cbl==id)
		{
			GtSRemove(&hd,&cbl->lnk);
			free(cbl);
			return;
		}
	}
}

/************************************************************************\
*
* Call functions on list.
*
\************************************************************************/
int GtCBList::CallCB(GtBase *obj,GtCDPtr cd)
{
	GtCBL *cbl,*tmp;
	int res;
	
	for(cbl=(GtCBL *)GtSFirst(&hd); cbl; cbl=tmp)
	{
		tmp=(GtCBL *)GtSNext(&cbl->lnk);
		res=cbl->cb(obj,cd,cbl->ud);
		if(res!=GTCB_CONT)
			return(res);
	}
	return(GTCB_CONT);
}
