#include <stdio.h>
#include <proto.h>

#include "gterror.h"

/************************************************************************\
*
* Error handling module.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Callback list for error handlers */
static GtCBList errcb;

/************************************************************************\
*
* Print error message on stderr.
*
\************************************************************************/
void GtPrintError(GtBase *obj,char *fmt,va_list va)
{
	if(obj)
		obj->PrtErrHeader();
	else
		fprintf(stderr,"Gt error:\n  ");
	#ifdef HAVE_VPRINTF
	vfprintf(stderr,fmt,va);
	#else
	fprintf(stderr,"%s",fmt);
	#endif
}

/************************************************************************\
*
* Print error message to stderr and exit.
*
\************************************************************************/
void GtError(GtBase *obj,ulong ecode,char *fmt,...)
{
	GtErrorCD cd;
	va_list va;
	
	va_start(va,fmt);
	cd.obj=obj;
	cd.ecode=ecode;
	cd.fmtstr=fmt;
	cd.valist=va;
	if(errcb.CallCB(NULL,&cd)==GTCB_BREAK)
	{
		va_end(va);
		return;
	}
	GtPrintError(obj,fmt,va);
	va_end(va);
	exit(1);
}

/************************************************************************\
*
* Add error callback.
*
\************************************************************************/
void GtAddErrorCB(GtCallB cb,GtUDPtr ud)
{
	errcb.AddCB(cb,ud);
}

/************************************************************************\
*
* Remove error callback.
*
\************************************************************************/
void GtRemErrorCB(GtCallB cb)
{
	errcb.RemCB(cb);
}
