#include <proto.h>

#include "gtbutton.h"
#include "gtappshell.h"
#include "gtmenu.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"
#include "gtconfig.h"

/************************************************************************\
*
* GtButton class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Button to button mask table */
static const btom[]={ 0, Button1Mask,Button2Mask,Button3Mask,
											Button4Mask,Button5Mask };
											
/************************************************************************\
*
* Create and init button.
*
\************************************************************************/
GtButton *GtMkButton(char *n,GtManager *p,GtTag *tgs)
{
	GtButton *obj;
	
	if(!(obj=new GtButton()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtButton object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init button.
*
\************************************************************************/
GtButton *GtMkButton(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtButton *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkButton(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty button.
*
\************************************************************************/
GtButton::GtButton(void)
{
	mshell=NULL;
	shortcut=NULL;
	scid=0;
	delay=500;
	idelay=500;
	bmask=Button1Mask;
	bflags=0;
	mclk=1;
	last=0;
	ibdrw=2;
	bpress=0;
}

/************************************************************************\
*
* Destroy button.
*
\************************************************************************/
void GtButton::Destroy(void)
{
	GtManager *pr;
	
	if((bflags & GTBF_REPEAT) && !mshell)
		app->RemTimeOut(GtButton::TimeOut);
	if(scid)
	{
		pr=parent;
		while(pr && !pr->IsSubClassOf("GtAppShell"))
			pr=pr->GetParent();
		if(pr)
			((GtAppShell *)pr)->RemKeyCB(scid);
	}
	GtGroup::Destroy();
}

/************************************************************************\
*
* Setup button.
*
\************************************************************************/
sbool GtButton::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	GtManager *pr;
	
	if(!GtGroup::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_DELAY,GTRT_LONG,&delay);
	app->GetRes(this,GTR_IDELAY,GTRT_LONG,&idelay);
	app->GetRes(this,GTR_SHORTCUT,GTRT_STRING,&shortcut);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Repeat:
				if(t->d.val)
					bflags|=GTBF_REPEAT;
				else
					bflags&=~GTBF_REPEAT;
				break;
			case GT_Delay:
				delay=t->d.val;
				break;
			case GT_InitialDelay:
				idelay=t->d.val;
				break;
			case GT_Shortcut:
				shortcut=(char *)t->d.ptr;
				break;
			case GT_ButtonMask:
				bmask=t->d.val;
				break;
		}
	}
	ibw=ibdrw+bdrw;
	ftype=GTF_OUT;
	if(shortcut)
	{
		for(pr=parent; pr; pr=pr->GetParent())
		{
			if(pr->IsSubClassOf("GtAppShell"))
			{
				scid=((GtAppShell *)pr)->AddKeyCB(shortcut,GtButton::ShortCut,this);
				break;
			}
		}
	}
	for(pr=parent; pr; pr=pr->GetParent())
	{
		if(pr->IsSubClassOf("GtMenu") || pr->IsSubClassOf("GtMenuBar"))
		{
			mshell=(GtMenu *)pr;
			bmask|=GTDEF_MENUBMASK;
			break;
		}
	}
	return(TRUE);
}

/************************************************************************\
*
* Set button attributes.
*
\************************************************************************/
sbool GtButton::Set(GtTag *tgs)
{
	GtTag *t;
	sbool ch=FALSE;
	
	if(!GtGroup::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Repeat:
				if(t->d.val)
					bflags|=GTBF_REPEAT;
				else
					bflags&=~GTBF_REPEAT;
				break;
			case GT_Delay:
				delay=t->d.val;
				break;
			case GT_InitialDelay:
				idelay=t->d.val;
				break;
			case GT_ButtonMask:
				bmask=t->d.val;
				break;
		}
	}
	if(ch)
	{
		ibw=ibdrw+bdrw;
		this->Relayout();
		this->Refresh();
	}
	return(TRUE);
}
	
/************************************************************************\
*
* Get button attributes.
*
\************************************************************************/
void GtButton::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtGroup::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Repeat:
				if(bflags & GTBF_REPEAT)
					*ptr=TRUE;
				else
					*ptr=FALSE;
				break;
			case GT_Delay:
				*ptr=delay;
				break;
			case GT_InitialDelay:
				*ptr=idelay;
				break;
		}
	}
}
	
/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtButton::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtButton")==0)
		return(TRUE);
	else
		return(GtGroup::IsSubClassOf(cn));
}

/************************************************************************\
*
* Set button event mask.
*
\************************************************************************/
void GtButton::SetEMask(ulong em)
{
	GtGroup::SetEMask(em|ExposureMask|ButtonPressMask|ButtonReleaseMask|
										EnterWindowMask|LeaveWindowMask);
}

/************************************************************************\
*
* Add callback.
*
\************************************************************************/
void GtButton::AddCB(ulong tag,GtCallB cb,GtUDPtr ud)
{
	switch(tag)
	{
		case GT_Arm:
			arm.AddCB(cb,ud);
			break;
		case GT_Activate:
			act.AddCB(cb,ud);
			break;
		case GT_Disarm:
			darm.AddCB(cb,ud);
			break;
		default:
			GtGroup::AddCB(tag,cb,ud);
			break;
	}
}

/************************************************************************\
*
* Remove callback.
*
\************************************************************************/
void GtButton::RemCB(ulong tag,GtCallB cb)
{
	switch(tag)
	{
		case GT_Arm:
			arm.RemCB(cb);
			break;
		case GT_Activate:
			act.RemCB(cb);
			break;
		case GT_Disarm:
			darm.RemCB(cb);
			break;
		default:
			GtGroup::RemCB(tag,cb);
			break;
	}
}

/************************************************************************\
*
* Check for activate button notify.
*
\************************************************************************/
void GtButton::Notify(ulong tag)
{
	GtButtonCD cd;
	
	if(tag==GT_Activate)
	{
		if(!(flags & GTWF_DISABLED))
		{
			cd.button=0;
			cd.state=0;
			cd.multi=0;
			act.CallCB(this,&cd);
			if((bflags & GTBF_ARMED) && (bflags & GTBF_REPEAT))
				app->AddTimeOut(delay,GtButton::TimeOut,this);
		}
	}
	else
		GtGroup::Notify(tag);
}

/************************************************************************\
*
* Draw button.
*
\************************************************************************/
sbool GtButton::Draw(XEvent *xe)
{
	if(xe && ((xe->type!=Expose) || (xe->xexpose.count!=0)))
		return(TRUE);
	if((bflags & GTBF_ARMED) && !mshell)
	{
		ftype=GTF_IN;
		GtGroup::Draw(NULL);
	}
	else if(((bflags & GTBF_ARMED) && mshell) || (!(bflags & GTBF_ARMED) && !mshell))
	{
		ftype=GTF_OUT;
		GtGroup::Draw(NULL);
	}
	return(TRUE);
}

/************************************************************************\
*
* Mouse button.
*
\************************************************************************/
sbool GtButton::Mouse(XEvent *xe)
{
	int b,s;
	ubyte tmp;
	
	if(!xe)
		return(FALSE);
	switch(xe->type)
	{
		case ButtonPress:
			b=xe->xbutton.button;
			if(flags & GTWF_DISABLED)
				break;
			if(!(bflags & GTBF_PRESS) && (bmask & btom[b]))
			{
				bpress=b;
				bflags|=(GTBF_ARMED|GTBF_PRESS);
				this->Refresh();
				this->Arm(b,xe->xbutton.state);
				return(TRUE);
			}
			break;
		case ButtonRelease:
			b=xe->xbutton.button;
			if((bflags & GTBF_PRESS) &&
				((b==bpress) || (mshell && (bmask & btom[b]))))
			{
				tmp=bflags & GTBF_ARMED;
				if(b==bpress)
				{
					bflags&=~(GTBF_ARMED|GTBF_PRESS);
					bpress=0;
				}
				this->Refresh();
				if(xe->xbutton.time-last>(ulong)dsp->GetMCTime())
					mclk=1;
				else
					mclk++;
				last=xe->xbutton.time;
				if(tmp)
				{
					this->Activate(b,xe->xbutton.state);
				}
				return(TRUE);
			}
			break;
		case EnterNotify:
			if((flags & GTWF_DISABLED) || (xe->xcrossing.mode!=NotifyNormal))
				break;
			s=xe->xcrossing.state;
			if(mshell && (s & bmask))
				bflags|=GTBF_PRESS;
			if((bflags & GTBF_PRESS) && !(bflags & GTBF_ARMED))
			{
				if(mshell)
					mshell->NotifyA(GT_Popdown,(void *)CurrentTime);
				bflags|=GTBF_ARMED;
				this->Refresh();
				this->Arm(bpress,s);
				return(TRUE);
			}
			break;
		case LeaveNotify:
			if((xe->xcrossing.mode!=NotifyNormal) ||
				(xe->xcrossing.detail==NotifyInferior))
				break;
			s=xe->xcrossing.state;
			if(bflags & GTBF_ARMED)
			{
				bflags&=~GTBF_ARMED;
				this->Refresh();
				this->DisArm(bpress,s);
				return(TRUE);
			}
			break;
	}
	return(FALSE);
}

/************************************************************************\
*
* Arm button.
*
\************************************************************************/
void GtButton::Arm(ulong button,ulong state)
{
	GtButtonCD cd;
	
	cd.button=button;
	cd.state=state;
	cd.multi=0;
	arm.CallCB(this,&cd);
	if((bflags & GTBF_REPEAT) && !mshell)
		app->AddTimeOut(idelay,GtButton::TimeOut,this);
}
		
/************************************************************************\
*
* Activate button.
*
\************************************************************************/
void GtButton::Activate(ulong button,ulong state)
{
	GtButtonCD cd;

	cd.button=button;
	cd.state=state;
	cd.multi=mclk;
	act.CallCB(this,&cd);
	darm.CallCB(this,&cd);
	if((bflags & GTBF_REPEAT) && !mshell)
		app->RemTimeOut(GtButton::TimeOut);
}

/************************************************************************\
*
* Disarm button.
*
\************************************************************************/
void GtButton::DisArm(ulong button,ulong state)
{
	GtButtonCD cd;
	
	cd.button=button;
	cd.state=state;
	cd.multi=0;
	darm.CallCB(this,&cd);
	if((bflags & GTBF_REPEAT) && !mshell)
		app->RemTimeOut(GtButton::TimeOut);
}

/************************************************************************\
*
* Timeout callback for button.
*
\************************************************************************/
int GtButton::TimeOut(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	obj=obj;
	cd=cd;
	((GtButton *)ud)->Notify(GT_Activate);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Shortcut callback for button.
*
\************************************************************************/
int GtButton::ShortCut(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	obj=obj;
	cd=cd;
	((GtButton *)ud)->Notify(GT_Activate);
	return(GTCB_CONT);
}
