#include <proto.h>

#include "gtcanvas.h"
#include "gterror.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtCanvas class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init canvas.
*
\************************************************************************/
GtCanvas *GtMkCanvas(char *n,GtManager *p,GtTag *tgs)
{
	GtCanvas *obj;
	
	if(!(obj=new GtCanvas()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtCanvas object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init canvas.
*
\************************************************************************/
GtCanvas *GtMkCanvas(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtCanvas *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkCanvas(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Set canvas event mask.
*
\************************************************************************/
void GtCanvas::SetEMask(ulong em)
{
	GtManager::SetEMask(em|KeyPressMask|KeyReleaseMask|ButtonPressMask|
												ButtonReleaseMask|ExposureMask);
}

/************************************************************************\
*
* Add callback function.
*
\************************************************************************/
void GtCanvas::AddCB(ulong tag,GtCallB cb,GtUDPtr ud)
{
	switch(tag)
	{
		case GT_Draw:
			draw.AddCB(cb,ud);
			break;
		case GT_Mouse:
			mouse.AddCB(cb,ud);
			break;
		case GT_Key:
			key.AddCB(cb,ud);
			break;
		case GT_Resize:
			resize.AddCB(cb,ud);
			break;
		default:
			GtManager::AddCB(tag,cb,ud);
			break;
	}
}

/************************************************************************\
*
* Remove callback function.
*
\************************************************************************/
void GtCanvas::RemCB(ulong tag,GtCallB cb)
{
	switch(tag)
	{
		case GT_Draw:
			draw.RemCB(cb);
			break;
		case GT_Mouse:
			mouse.RemCB(cb);
			break;
		case GT_Key:
			key.RemCB(cb);
			break;
		case GT_Resize:
			resize.RemCB(cb);
			break;
		default:
			GtManager::RemCB(tag,cb);
			break;
	}
}

/************************************************************************\
*
* Draw canvas.
*
\************************************************************************/
sbool GtCanvas::Draw(XEvent *xe)
{
	GtCanvasCD cd;
	
	cd.xevent=xe;
	cd.gfx=gfx;
	cd.xsize=xsize;
	cd.ysize=ysize;
	cd.used=TRUE;
	draw.CallCB(this,&cd);
	return(cd.used);
}

/************************************************************************\
*
* Mouse events.
*
\************************************************************************/
sbool GtCanvas::Mouse(XEvent *xe)
{
	GtCanvasCD cd;
	
	cd.xevent=xe;
	cd.gfx=gfx;
	cd.xsize=xsize;
	cd.ysize=ysize;
	cd.used=TRUE;
	mouse.CallCB(this,&cd);
	return(cd.used);
}

/************************************************************************\
*
* Key events.
*
\************************************************************************/
sbool GtCanvas::Key(XEvent *xe)
{
	GtCanvasCD cd;
	
	cd.xevent=xe;
	cd.gfx=gfx;
	cd.xsize=xsize;
	cd.ysize=ysize;
	cd.used=FALSE;
	key.CallCB(this,&cd);
	return(cd.used);
}

/************************************************************************\
*
* Resized canvas.
*
\************************************************************************/
void GtCanvas::Layout(void)
{
	GtCanvasCD cd;
	
	if(flags & GTWF_UNBUILT)
		return;
	cd.xevent=NULL;
	cd.gfx=gfx;
	cd.xsize=xsize;
	cd.ysize=ysize;
	cd.used=TRUE;
	resize.CallCB(this,&cd);
}
