#include <proto.h>

#include "gtlabel.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtLabel class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

#define GTLT_STRING		0
#define GTLT_PIXMAP		1

typedef struct GtStrPart
{
	char *str;
	int len,w;
} GtStrPart;

/************************************************************************\
*
* Create and init label.
*
\************************************************************************/
GtLabel *GtMkLabel(char *n,GtManager *p,GtTag *tgs)
{
	GtLabel *obj;
	
	if(!(obj=new GtLabel()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtLabel object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init label.
*
\************************************************************************/
GtLabel *GtMkLabel(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtLabel *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkLabel(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty label.
*
\************************************************************************/
GtLabel::GtLabel(void)
{
	ld.str=NULL;
	ltype=GTLT_STRING;
	lalign=GTA_CENTER;
	ulch=0;
	lppix=FALSE;
	lw=1;
	lh=1;
	lrows=1;
}

/************************************************************************\
*
* Destroy label.
*
\************************************************************************/
void GtLabel::Destroy(void)
{
	if(ltype==GTLT_STRING)
	{
		if(ld.str)
			free(ld.str);
	}
	if(ltype==GTLT_PIXMAP)
	{
		if(ld.pix && lppix)
			GtDestroy(ld.pix);
	}
	GtWidget::Destroy();
}

/************************************************************************\
*
* Setup label.
*
\************************************************************************/
sbool GtLabel::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	char *s=NULL,*pn=NULL,*uch=NULL;
	sbool bm=FALSE,fulch=TRUE;
	int i,l,r;
	XCharStruct xcs;
	
	if(!GtWidget::Init(n,p,tgs))
		return(FALSE);
	if(app->GetRes(this,GTR_STRING,GTRT_STRING,&s))
		ltype=GTLT_STRING;
	if(GtXpmSupport() && app->GetRes(this,GTR_XPM,GTRT_STRING,&pn))
	{
		ld.pix=GtMkPixmap(dsp,
			GT_XpmName,pn,
			GT_Depth,gfx->depth,
			GT_Background,gfx->cols[GTBACK],
			GT_DONE);
		ltype=GTLT_PIXMAP;
		lppix=TRUE;
		bm=FALSE;
	}
	else if(app->GetRes(this,GTR_BITMAP,GTRT_STRING,&pn))
	{
		ld.pix=GtMkPixmap(dsp,GT_BitmapName,pn,GT_DONE);
		ltype=GTLT_PIXMAP;
		lppix=TRUE;
		bm=TRUE;
	}
	app->GetRes(this,GTR_ALIGN,GTRT_ALIGN,&lalign);
	if(app->GetRes(this,GTR_ULINE,GTRT_STRING,&uch) && uch)
	{
		ulch=uch[0];
	}
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_String:
				s=(char *)t->d.ptr;
				ltype=GTLT_STRING;
				break;
			case GT_Pixmap:
				if((ltype==GTLT_PIXMAP) && ld.pix && lppix)
					GtDestroy(ld.pix);
				lppix=FALSE;
				ld.pix=(GtPixmap *)t->d.ptr;
				ltype=GTLT_PIXMAP;
				break;
			case GT_Align:
				lalign=t->d.val;
				if(lalign>GTA_LAST_NR)
					lalign=GTA_CENTER;
				break;
			case GT_UnderLineChar:
				ulch=t->d.val;
				break;
		}
	}
	if(ltype==GTLT_STRING)
	{
		if(!s)
			s=name;
		lrows=1;
		for(i=0; s[i]; i++)
		{
			if(s[i]=='\n')
				lrows++;
		}
		if(!(ld.str=(GtStrPart *)malloc(sizeof(GtStrPart)*lrows)))
		{
			GtError(this,GTEC_NOMEMORY,"No memory for string parts!\n");
			return(FALSE);
		}
		l=0;
		r=0;
		ld.str[r].str=s;
		for(i=0; s[i]; i++)
		{
			if((s[i]==ulch) && fulch)
			{
				ulxp=l;
				ulyp=r;
				fulch=FALSE;
			}
			l++;
			if(s[i]=='\n')
			{
				ld.str[r].len=l-1;
				ld.str[r].w=gtfnt->StrWidth(ld.str[r].str,ld.str[r].len);
				lw=GTMAX(lw,ld.str[r].w);
				r++;
				l=0;
				ld.str[r].str=&s[i+1];
			}
		}
		ld.str[r].len=l;
		ld.str[r].w=gtfnt->StrWidth(ld.str[r].str,ld.str[r].len);
		lw=GTMAX(lw,ld.str[r].w);
		gtfnt->MaxExtents(&xcs);
		loff=xcs.ascent;
		lh=loff+xcs.descent;
		if(ulch)
		{
			ulw=gtfnt->CharWidth(ulch);
			if(ulxp)
				ulxp=gtfnt->StrWidth(ld.str[ulyp].str,ulxp);
		}
	}
	else if((ltype==GTLT_PIXMAP) && ld.pix)
	{
		ld.pix->GetV(GT_XSize,&l,GT_YSize,&r,GT_DONE);
		lw=l;
		lh=r;
		lrows=1;
	}
	this->SetMinMaxSize(GTMAX(lw,1),GTMAX(lh*lrows,1),GTS_MAX,GTS_MAX);
	return(TRUE);
}

/************************************************************************\
*
* Set label attributes.
*
\************************************************************************/
sbool GtLabel::Set(GtTag *tgs)
{
	GtTag *t;
	char *s=NULL;
	sbool ch=FALSE,vch=FALSE,fch=FALSE,fulch=TRUE;
	int i,l,r;
	XCharStruct xcs;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_String:
				if(!t->d.ptr)
					break;
				if((ltype==GTLT_PIXMAP) && ld.pix && lppix)
					GtDestroy(ld.pix);
				lppix=FALSE;
				if((ltype==GTLT_STRING) && ld.str)
					free(ld.str);
				ld.str=NULL;
				s=(char *)t->d.ptr;
				ltype=GTLT_STRING;
				ch=TRUE;
				break;
			case GT_Pixmap:
				if((ltype==GTLT_PIXMAP) && ld.pix && lppix)
					GtDestroy(ld.pix);
				lppix=FALSE;
				if((ltype==GTLT_STRING) && ld.str)
					free(ld.str);
				ld.str=NULL;
				ld.pix=(GtPixmap *)t->d.ptr;
				ltype=GTLT_PIXMAP;
				ch=TRUE;
				break;
			case GT_Align:
				lalign=t->d.val;
				if(lalign>GTA_LAST_NR)
					lalign=GTA_CENTER;
				vch=TRUE;
				break;
			case GT_UnderLineChar:
				ulch=t->d.val;
				ch=TRUE;
				break;
			case GT_Font:
			case GT_FontList:
				fch=TRUE;
				break;
		}
	}
	if(fch || ch)
	{
		lw=1;
		lh=1;
	}
	if((ltype==GTLT_STRING) && (fch || ch))
	{
		if(ch)
		{
			lrows=1;
			for(i=0; s[i]; i++)
			{
				if(s[i]=='\n')
					lrows++;
			}
			if(!(ld.str=(GtStrPart *)malloc(sizeof(GtStrPart)*lrows)))
			{
				GtError(this,GTEC_NOOBJECT,"No memory for string parts!\n");
				return(FALSE);
			}
		}
		l=0;
		r=0;
		ld.str[r].str=s;
		for(i=0; s[i]; i++)
		{
			if((s[i]==ulch) && fulch)
			{
				ulxp=l;
				ulyp=r;
				fulch=FALSE;
			}
			l++;
			if(s[i]=='\n')
			{
				ld.str[r].len=l-1;
				ld.str[r].w=gtfnt->StrWidth(ld.str[r].str,ld.str[r].len);
				lw=GTMAX(lw,ld.str[r].w);
				r++;
				l=0;
				ld.str[r].str=&s[i+1];
			}
		}
		ld.str[r].len=l;
		ld.str[r].w=gtfnt->StrWidth(ld.str[r].str,ld.str[r].len);
		lw=GTMAX(lw,ld.str[r].w);
		gtfnt->MaxExtents(&xcs);
		lh=loff+xcs.descent;
		if(ulch)
		{
			ulw=gtfnt->CharWidth(ulch);
			if(ulxp)
				ulxp=gtfnt->StrWidth(ld.str[ulyp].str,ulxp);
		}
	}
	else if((ltype==GTLT_PIXMAP) && ld.pix && ch)
	{
		ld.pix->GetV(GT_XSize,&l,GT_YSize,&r,GT_DONE);
		lw=l;
		lh=r;
		lrows=1;
	}
	if(ch || fch)
		this->SetMinMaxSize(GTMAX(lw,1),GTMAX(lh*lrows,1),GTS_MAX,GTS_MAX);
	if(ch || fch || vch)
		this->Refresh();
	return(TRUE);
}
	
	
/************************************************************************\
*
* Get label attributes.
*
\************************************************************************/
void GtLabel::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_String:
				if(ltype==GTLT_STRING)
					*ptr=(long)ld.str[0].str;
				break;
			case GT_Pixmap:
				if(ltype==GTLT_PIXMAP)
					*ptr=(long)ld.pix;
				break;
			case GT_Align:
				*ptr=lalign;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtLabel::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtLabel")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* Set label event mask.
*
\************************************************************************/
void GtLabel::SetEMask(ulong em)
{
	GtWidget::SetEMask(em|ExposureMask);
}

/************************************************************************\
*
* Draw label when expose.
*
\************************************************************************/
sbool GtLabel::Draw(XEvent *xe)
{
	int n,xp=0,yp=0;

	if(xe && ((xe->type!=Expose) || (xe->xexpose.count!=0)))
		return(TRUE);
	if(ltype==GTLT_STRING)
	{
		if(!ld.str)
			return(TRUE);
		yp=(ysize-lh*lrows)/2;
		if(flags & GTWF_NOWIN)
			yp+=ypos;
		for(n=0; n<lrows; n++)
		{
			if(lalign==GTA_CENTER)
				xp=(xsize-ld.str[n].w)/2;
			else if(lalign==GTA_RIGHT)
				xp=xsize-ld.str[n].w;
			else
				xp=0;
			if(flags & GTWF_NOWIN)
				xp+=xpos;
			gtfore->DrawString(gtwin,xp,loff+yp,ld.str[n].str,ld.str[n].len,FALSE);
			if(ulch && (ulyp==n))
				gtfore->DrawLine(gtwin,xp+ulxp,yp+loff+1,xp+ulxp+ulw-1,yp+loff+1);
			yp+=lh;
		}
	}
	else if(ltype==GTLT_PIXMAP)
	{
		if(!ld.pix)
			return(TRUE);
		if(lalign==GTA_CENTER)
			xp=(xsize-lw)/2;
		else if(lalign==GTA_RIGHT)
			xp=xsize-lw;
		else
			xp=0;
		yp=(ysize-lh)/2;
		if(flags & GTWF_NOWIN)
		{
			xp+=xpos;
			yp+=ypos;
		}
		if(ld.pix->IsBitmap())
			gtfore->CopyPlane(ld.pix,gtwin,0,0,lw,lh,xp,yp,1);
		else
			gtfore->CopyArea(ld.pix,gtwin,0,0,lw,lh,xp,yp);
	}
	return(TRUE);
}
